from hunabku.HunabkuBase import HunabkuPluginBase, endpoint
from bson import ObjectId


class MoaiGSLookUp(HunabkuPluginBase):
    def __init__(self, hunabku):
        super().__init__(hunabku)

    @endpoint('/moai/gs/lookup/data', methods=['GET'])
    def data_endpoint(self):
        """
        @api {get} /moai/gs/lookup/data Retrieve data by ids
        @apiName GSLookUp
        @apiGroup Moai GSLookUp
        @apiDescription Allow to download the registers from collection data for given ids

        @apiParam {String} db  Database to use in mongodb
        @apiParam {String} ids  Paper ids to retrieve
        @apiParam {String} apikey  Credential for authentication


        @apiSuccess {Object}  json  all the registers from data collection in a json dump

        @apiError (Error 401) msg  The HTTP 401 Unauthorized invalid authentication apikey for the target resource.
        """
        ids = self.json.loads(self.request.args.get('ids').replace("'", "\""))
        oids = [ObjectId(iid) for iid in ids]
        db = self.request.args.get('db')
        self.db = self.dbclient[db]

        if self.valid_apikey():
            cursor = self.db['data'].find({'_id': {'$in': oids}})
            data = []
            for i in cursor:
                data.append(i)
            response = self.app.response_class(
                response=self.json.dumps(data),
                status=200,
                mimetype='application/json'
            )
            return response
        else:
            return self.apikey_error()

    @endpoint('/moai/gs/lookup/not_found', methods=['GET'])
    def not_found(self):
        """
        @api {get} /moai/gs/lookup/not_found GSLookUp not found
        @apiName GSLookUp
        @apiGroup Moai GSLookUp
        @apiDescription Allow to move the register from data when not found for gslookup to the collection gslookup_not_found

        @apiParam {String} db  Database to use in mongodb
        @apiParam {String} id  Paper id to move
        @apiParam {String} apikey  Credential for authentication


        @apiSuccess {String}  msg  Message

        @apiError (Error 401) msg  The HTTP 401 Unauthorized invalid authentication apikey for the target resource.
        """

        _id = self.request.args.get('_id')
        oid = ObjectId(_id)
        db = self.request.args.get('db')
        url = self.request.args.get('url')

        if self.valid_apikey():
            self.db = self.dbclient[db]
            cursor = self.db['data'].find({'_id': oid})
            data = []
            for i in cursor:
                data.append(i)
            data[0]['url'] = url
            self.db['gslookup_not_found'].insert(data)
            self.db['data'].delete_one({'_id': oid})
            response = self.app.response_class(
                response=self.json.dumps(
                    {'msg': 'register {} moved from data to gslookup_not_data'.format(_id)}),
                status=200,
                mimetype='application/json'
            )
            return response
        else:
            return self.apikey_error()

    @endpoint('/moai/gs/lookup/checkpoint', methods=['GET'])
    def stage_checkpoint(self):
        """
        @api {get} /moai/gs/lookup/checkpoint GSLookUp checkpoint
        @apiName GSLookUp
        @apiGroup Moai GSLookUp
        @apiDescription Allow to know the cuerrent status of the collection data for the given dataset in db
                        Return the ids of papers not dowloaded yet comparing the ids from data and stage collections using set.

        @apiParam {String} db  Database to use in mongodb
        @apiParam {String} apikey  Credential for authentication


        @apiSuccess {Bool}  checkpoint  if true then there are more ids to download
        @apiSuccess {String[]}  ids if true then there are more ids to download
        @apiSuccess {Bool}  error  if true then there is an error to handle the ids ex: not collection data to get ids
        @apiSuccess {String} msg Message with the explanation of the error in case error tag is true.

        @apiError (Error 401) msg  The HTTP 401 Unauthorized invalid authentication apikey for the target resource.
        """
        if self.valid_apikey():
            db = self.request.args.get('db')
            self.db = self.dbclient[db]
            ckeckpoint = True  # False if any error or all was dowloaded
            error = False
            msg = ""
            ckp_ids = []  # _id(s) for checkpoint

            # reading collection data
            try:
                data_ids = set([str(reg["_id"])
                                for reg in self.db['data'].find({}, {"_id": 1})])
            except BaseException:
                data_ids = []
            npapers = len(data_ids)
            if npapers == 0:
                error = True
                ckeckpoint = False
                msg = "No elements found in database " + db + ' collection data'
                response = self.app.response_class(
                    response=self.json.dumps(
                        {'checkpoint': ckeckpoint, 'ids': ckp_ids, 'error': error, 'msg': msg}),
                    status=200,
                    mimetype='application/json'
                )
                return response

            try:
                stage_ids = set([str(reg["_id"])
                                 for reg in self.db['stage'].find({}, {'_id': 1})])
            except BaseException:
                stage_ids = []

            try:
                quarantine_ids = set(
                    [str(reg["_id"]) for reg in self.db['quarantine'].find({}, {'_id': 1})])
            except BaseException:
                quarantine_ids = []

            if len(stage_ids) + \
                    len(quarantine_ids) == npapers:  # all the papers were downloaded
                ckeckpoint = False
                msg = "All papers already downloaded for database " + db + ' collection data'
                response = self.app.response_class(
                    response=self.json.dumps(
                        {'checkpoint': ckeckpoint, 'ids': ckp_ids, 'error': error, 'msg': msg}),
                    status=200,
                    mimetype='application/json'
                )
                return response

            if len(stage_ids) == 0 and len(quarantine_ids) == 0:
                ckp_ids = list(data_ids)
                msg = 'stage and quarantine in database ' + db + ' is empty'
                response = self.app.response_class(
                    response=self.json.dumps(
                        {'checkpoint': ckeckpoint, 'ids': ckp_ids, 'error': error, 'msg': msg}),
                    status=200,
                    mimetype='application/json'
                )
                return response

            ckp_ids = list(data_ids - data_ids.intersection(stage_ids) - data_ids.intersection(quarantine_ids))
            msg = 'missing values for stage and quarantine with database ' + db + ' collection data'
            response = self.app.response_class(
                response=self.json.dumps(
                    {'checkpoint': ckeckpoint, 'ids': ckp_ids, 'error': error, 'msg': msg}),
                status=200,
                mimetype='application/json'
            )
            return response

        else:
            return self.apikey_error()

    @endpoint('/moai/gs/lookup/quarantine', methods=['GET'])
    def quarantine_submit(self):
        """
        @api {get} /moai/gs/lookup/quarantine Submit Paper in Quarantine collection
        @apiName  Moai GSLookUp
        @apiGroup Moai GSLookUp
        @apiDescription Allows to submit papers to the collection quarantine in the given databse db.

        @apiParam {String} db  Database to use in mongodb
        @apiParam {Object} data Json with paper data
        @apiParam {String} apikey  Credential for authentication


        @apiSuccess {String}  msg  GSLookUp paper inserted in quarantine
        @apiError (Error 401) msg  The HTTP 401 Unauthorized invalid authentication apikey for the target resource.
        """
        data = self.request.args.get('data')
        db = self.request.args.get('db')
        self.db = self.dbclient[db]
        if self.valid_apikey():
            jdata = self.json.loads(data)
            jdata["_id"] = ObjectId(jdata["_id"])
            self.db['quarantine'].insert(jdata, check_keys=False)
            response = self.app.response_class(
                response=self.json.dumps(
                    {'msg': 'GSLookUp Paper inserted in quarantine'}),
                status=200,
                mimetype='application/json'
            )
            return response
        else:
            return self.apikey_error()



from hunabku.HunabkuBase import HunabkuPluginBase, endpoint


class MoaiGSProfile(HunabkuPluginBase):
    def __init__(self, hunabku):
        super().__init__(hunabku)

    @endpoint('/moai/gs/profile/checkpoint', methods=['GET'])
    def profile_checkpoint(self):
        """
        @api {get} /moai/gs/profile/checkpoint GSProfile checkpoint
        @apiName GSProfile
        @apiGroup Moai GSProfile
        @apiDescription Allow to know the cuerrent status of the collection data for the given dataset in db
                        Return the ids of papers not dowloaded yet comparing the ids from data and stage collections using set.

        @apiParam {String} db  Database to use in mongodb
        @apiParam {String} apikey  Credential for authentication


        @apiSuccess {Bool}  checkpoint  if true then there are more ids to download
        @apiSuccess {String[]}  ids if true then there are more ids to download
        @apiSuccess {Bool}  error  if true then there is an error to handle the ids ex: not collection data to get ids
        @apiSuccess {String} msg Message with the explanation of the error in case error tag is true.

        @apiError (Error 401) msg  The HTTP 401 Unauthorized invalid authentication apikey for the target resource.
        """
        if self.valid_apikey():
            db = self.request.args.get('db')
            self.db = self.dbclient[db]
            stage = self.db['stage']
            profiles = self.db['profiles']
            ckeckpoint = True  # False if any error or all was dowloaded
            error = False
            msg = ""
            ckp_ids = []  # _id(s) for checkpoint

            # reading collection data
            try:
                profiles_c = stage.find({'profiles': {'$ne': {}}}, {
                                        '_id': 0, 'profiles': 1})
                profiles_p = profiles.find({}, {'_id': 1})

                profiles_stage = list(profiles_c)
                profiles_stage_ids = []
                for profile in profiles_stage:
                    for user in profile['profiles']:
                        profiles_stage_ids.append(profile['profiles'][user])

                profiles_stage_ids = set(profiles_stage_ids)

                profiles_collection = list(profiles_p)
                profiles_ids = []
                for profile in profiles_collection:
                    profiles_ids.append(profile['_id'])
                profiles_ids = set(profiles_ids)
                ckp_ids = list(profiles_stage_ids - profiles_ids)

            except BaseException:
                ckp_ids = []

            msg = 'ids for profiles in database ' + db + \
                ' collection stage still not downloaded'
            response = self.app.response_class(
                response=self.json.dumps(
                    {'checkpoint': ckeckpoint, 'ids': ckp_ids, 'error': error, 'msg': msg}),
                status=200,
                mimetype='application/json'
            )
            return response

        else:
            return self.apikey_error()

    @endpoint('/moai/gs/profile/submit', methods=['GET'])
    def profile_submit(self):
        """
        @api {get} /moai/gs/profile/submit Submit profile
        @apiName  Moai profile
        @apiGroup Moai GSProfile
        @apiDescription Allows to submit papers to the collection profile in the given databse db.

        @apiParam {String} db  Database to use in mongodb
        @apiParam {Object} data Json with paper data
        @apiParam {String} apikey  Credential for authentication


        @apiSuccess {String}  msg  GSProifle profile inserted
        @apiError (Error 401) msg  The HTTP 401 Unauthorized invalid authentication apikey for the target resource.
        """
        data = self.request.args.get('data')
        db = self.request.args.get('db')
        self.db = self.dbclient[db]
        if self.valid_apikey():
            data = self.json.loads(data)
            self.db['profiles'].insert(data)
            response = self.app.response_class(
                response=self.json.dumps(
                    {'msg': 'GSProfile user inserted in profile'}),
                status=200,
                mimetype='application/json'
            )
            return response
        else:
            return self.apikey_error()

    @endpoint('/moai/gs/profile/not_found', methods=['GET'])
    def profile_not_found(self):
        """
        @api {get} /moai/gs/profile/not_found GSProfile not found
        @apiName GSProfile
        @apiGroup Moai GSProfile
        @apiDescription Allow to move the register from data when not found for gsprofile to the collection gsprofile_not_found

        @apiParam {String} db  Database to use in mongodb
        @apiParam {String} id  Paper id to move
        @apiParam {String} apikey  Credential for authentication


        @apiSuccess {String}  msg  Message

        @apiError (Error 401) msg  The HTTP 401 Unauthorized invalid authentication apikey for the target resource.
        """

        _id = self.request.args.get('_id')
        db = self.request.args.get('db')
        url = self.request.args.get('url')

        if self.valid_apikey():
            self.db = self.dbclient[db]
            self.db['gsprofile_not_found'].insert({'_id': _id, 'url': url})
            response = self.app.response_class(
                response=self.json.dumps(
                    {'msg': 'register {} moved from data to gsprofile_not_data'.format(_id)}),
                status=200,
                mimetype='application/json'
            )
            return response
        else:
            return self.apikey_error()




from flask import (
    json,
    request
)
from bson import ObjectId
from functools import wraps


class HunabkuJsonEncoder(json.JSONEncoder):
    """
    Custom JSON encoder for Hunabku,
    all the customized stuff for encoding required for our endpoints
    can be handle in this class
    """

    def default(self, o):
        if isinstance(o, ObjectId):
            return str(o)
        return json.JSONEncoder.default(self, o)


_endpoints = {}  # global hidden dictionary to register every endpoint by plugin


def endpoint(path, methods):
    """
    Specilaized decorator to use in the methods of the class that inherit from  HunabkuPluginBase
    this decorator allows to register the path and methods [GET,POST,DELETE,PUT]
    in the flask app.

    example:
    class Hello(HunabkuPluginBase):
    def __init__(self,hunabku):
        super().__init__(hunabku)

    @endpoint('/hello',methods=['GET'])
    def hello(self):
        if self.valid_apikey():
            response = self.app.response_class(
                response=self.json.dumps({'hello':'world'}),
                status=200,
                mimetype='application/json'
            )
            return response
        else:
            return self.send_apikey_error()

    """
    def wrapper(func):
        global _endpoints
        print('------ Adding endpoint ' + path + ' with methods' + str(methods))
        class_name, func_name = func.__qualname__.split('.')
        if class_name not in _endpoints:
            _endpoints[class_name] = []
        _endpoints[class_name].append(
            {'path': path, 'methods': methods, 'func_name': func_name})

        @wraps(func)
        def _impl(self, *method_args, **method_kwargs):
            response = func(self)
            return response
        return _impl
    return wrapper


class HunabkuPluginBase:
    def __init__(self, hunabku):
        """
        Base class to handle the plugins.
        Allows to have access to the MondoDB object, custom json methods
        with our encoders, utility functions to check apikeys and send default messages
        in case of error.

        This class allows to register the endpoints setting a decorator in the method
        that is going to hanlde the endpoint.

        """
        self.dburi = hunabku.dburi
        self.dbclient = hunabku.dbclient
        self.ip = hunabku.ip
        self.port = hunabku.port
        self.info_level = hunabku.info_level
        self.apikey = hunabku.apikey
        self.app = hunabku.app
        self.request = request
        self.json = json
        self.logger = hunabku.logger
        self.hunabku = hunabku
        _dumps = self.json.dumps
        _dump = self.json.dump

        # added support to our json encoder
        def json_dumps(
                obj,
                skipkeys=False,
                ensure_ascii=True,
                check_circular=True,
                allow_nan=True,
                cls=HunabkuJsonEncoder,
                indent=None,
                separators=None,
                default=None,
                sort_keys=False):
            return _dumps(
                obj=obj,
                skipkeys=skipkeys,
                ensure_ascii=ensure_ascii,
                check_circular=check_circular,
                allow_nan=allow_nan,
                cls=cls,
                indent=indent,
                separators=separators,
                default=default,
                sort_keys=sort_keys)

        def json_dump(
                obj,
                skipkeys=False,
                ensure_ascii=True,
                check_circular=True,
                allow_nan=True,
                cls=HunabkuJsonEncoder,
                indent=None,
                separators=None,
                default=None,
                sort_keys=False):
            return _dump(
                obj=obj,
                skipkeys=skipkeys,
                ensure_ascii=ensure_ascii,
                check_circular=check_circular,
                allow_nan=allow_nan,
                cls=cls,
                indent=indent,
                separators=separators,
                default=default,
                sort_keys=sort_keys)
        # custimized encoder use by default
        self.json.dumps = json_dumps
        self.json.dump = json_dump

    def apikey_error(self):
        """
        return defualt apikey error
        """
        response = self.app.response_class(
            response=self.json.dumps(
                {'msg': 'The HTTP 401 Unauthorized invalid authentication apikey for the target resource.'}),
            status=401,
            mimetype='application/json'
        )
        return response

    def valid_apikey(self):
        apikey = self.request.args.get('apikey')
        if self.apikey == apikey:
            return True
        else:
            return False

    def register_endpoints(self):
        """
        Method to register all the endpoints in flask's app
        """
        global _endpoints
        if self.is_valid_endpoints():
            for endpoint_data in _endpoints[type(self).__name__]:
                path = endpoint_data['path']
                func_name = endpoint_data['func_name']
                methods = endpoint_data['methods']
                func = getattr(self, func_name)
                self.app.add_url_rule(path, view_func=func, methods=methods)

    def get_global_endpoints(self):
        """
        Method to return the global dictionary with all
        the registers  loaded
        """
        return _endpoints

    def is_valid_endpoints(self):
        """
        This method checks before to load the plugin if any paths in the endpoint is repeated.
        this platform does not allows overwrite endpoint paths.
        """
        plugins = list(_endpoints.keys())
        current = type(self).__name__
        plugins.remove(current)
        paths = []
        for register in _endpoints[current]:
            paths.append(register['path'])
        for path in paths:
            for plugin in plugins:
                for register in _endpoints[plugin]:
                    if path == register['path']:
                        self.logger.error(
                            "ERROR: can't not load plugin, {} because the path {} is already loaded in plugin {}".format(
                                current, path, plugin))
                        return False
        return True



from flask import (
    Flask
)

from pymongo import MongoClient

import logging

import os
from hunabku._version import get_version
from shutil import rmtree
from distutils.dir_util import copy_tree
import subprocess
import glob
import time
import pathlib
import sys
import importlib
import json


class Hunabku:
    """
    Class to serve papers information store in a mongodb database throught an API using flask.

    example:
    http://0.0.0.0:5000/data/redalyc?init=1&end&apikey=pl0ok9ij8uh7yg
    """

    def __init__(
            self,
            apikey,
            dburi='mongodb://localhost:27017/',
            ip='127.0.0.1',
            port=8080,
            log_file='hunabku.log',
            info_level=logging.DEBUG):
        """
        Contructor to initialize configuration options.

        Args:
            apikey: apikey to access the data
            ip (str): ip to start the server
            port (int): port for the server
            info_level (logging.DEBUG/INFO etc..): enable/disable debug mode with extra messages output.
        """
        self.dburi = dburi
        self.dbclient = MongoClient(dburi)
        self.ip = ip
        self.port = port
        self.info_level = info_level
        self.apikey = apikey
        self.apidoc_dir = 'hunabku_website'
        self.apidoc_static_dir = self.apidoc_dir + '/static'
        self.apidoc_output_dir = self.apidoc_dir + '/static/apidoc'
        self.apidoc_templates_dir = self.apidoc_dir + '/templates'
        self.apidoc_config_dir = self.apidoc_dir + '/config'
        self.apidoc_config_data = {}
        self.apidoc_config_data['url'] = 'http://' + \
            ip + ':' + str(port) + '/apidoc'
        self.apidoc_config_data['sampleUrl'] = 'http://' + ip + ':' + str(port)
        self.apidoc_config_data['header'] = {}
        self.apidoc_config_data['header']['filename'] = self.apidoc_config_dir + \
            '/apidoc-header.md'
        self.apidoc_config_data['version'] = get_version()
        self.pkg_config_dir = str(
            pathlib.Path(__file__).parent.absolute()) + '/config/'
        self.pkg_templates_dir = str(
            pathlib.Path(__file__).parent.absolute()) + '/templates/'
        self.plugins = []
        self.log_file = log_file
        self.logger = logging.getLogger(__name__)
        self.set_info_level(info_level)
        self.app = Flask(
            'hanubku',
            static_folder=self.apidoc_static_dir,
            static_url_path='/',
            template_folder=self.apidoc_templates_dir)
        self.apidoc_setup()
        self.load_plugins()
        self.generate_doc()

    def apidoc_setup(self):
        """
        creates an ApiDoc folder to dump configuration and documentation of the APIs
        """
        try:
            os.makedirs(self.apidoc_dir, exist_ok=True)
            print(" * ApidDoc Directory ", self.apidoc_dir, " created ")
        except FileExistsError:
            # Is this is happening then restart the microservices in the folder
            print(
                " * Warning! ApidDoc Directory ",
                self.apidoc_dir,
                " already exists")

        try:
            os.mkdir(self.apidoc_static_dir)
            print(
                " * ApiDoc Static Directory ",
                self.apidoc_static_dir,
                " created ")
        except FileExistsError:
            # Is this is happening then restart the microservices in the folder
            print(
                " * Warning! ApiDoc Static Directory ",
                self.apidoc_static_dir,
                " already exists")

        try:
            os.mkdir(self.apidoc_output_dir)
            print(" * ApiDoc Static Output Directory ",
                  self.apidoc_output_dir, " created ")
        except FileExistsError:
            # Is this is happening then restart the microservices in the folder
            print(
                " * Warning! ApiDoc Static Output Directory ",
                self.apidoc_output_dir,
                " already exists")

        try:
            os.mkdir(self.apidoc_templates_dir)
            print(
                " * ApiDoc Templates Directory ",
                self.apidoc_templates_dir,
                " created ")
        except FileExistsError:
            # Is this is happening then restart the microservices in the folder
            print(
                " * Warning! ApiDoc Templates Directory ",
                self.apidoc_templates_dir,
                " already exists")

        try:
            os.mkdir(self.apidoc_config_dir)
            print(
                " * ApiDoc Config Directory ",
                self.apidoc_static_dir,
                " created ")
        except FileExistsError:
            # Is this is happening then restart the microservices in the folder
            print(
                " * Warning! ApiDoc Static Directory ",
                self.apidoc_static_dir,
                " already exists")

        copy_tree(self.pkg_config_dir, self.apidoc_config_dir)
        apidoc_config_data = {}
        with open(self.apidoc_config_dir + '/apidoc.json') as json_file:
            apidoc_config_data = json.load(json_file)

        apidoc_config_data.update(self.apidoc_config_data)
        with open(self.apidoc_config_dir + '/apidoc.json', 'w') as json_file:
            json.dump(apidoc_config_data, json_file)

        copy_tree(self.pkg_templates_dir, self.apidoc_templates_dir)

    def set_info_level(self, info_level):
        """
        Information level for debug or verbosity of the application (https://docs.python.org/3.1/library/logging.html)
        """
        if info_level != logging.DEBUG:
            logging.basicConfig(filename=self.log_file, level=info_level)
        self.info_level = info_level

    def load_plugins(self):
        """
        This method return the plugins found in the folder plugins.
        """
        self.logger.warning('-----------------------')
        self.logger.warning('------ Loagind Plugins:')
        for path in glob.glob(
                str(pathlib.Path(__file__).parent.absolute()) + "/plugins/*.py"):
            name = path.split(os.path.sep)[-1].replace('.py', '')
            print('------ Loading plugin: ' + name)
            spec = importlib.util.spec_from_file_location(name, path)
            module = spec.loader.load_module()
            plugin_class = getattr(module, name)
            instance = plugin_class(self)
            instance.register_endpoints()
            plugin = {}
            plugin['name'] = name
            plugin['path'] = path
            plugin['spec'] = spec
            plugin['instance'] = instance
            self.plugins.append(plugin)

    def check_apidoc_syntax(self, plugin_file):
        """
        Allows to check in the syntaxis in the docstring comment is right
        for apidoc  files generation.
        The the syntax is wrong, the Hunabku server can not start.
        """
        args = ['apidoc', '-c', self.apidoc_config_dir, '-i',
                str(pathlib.Path(__file__).parent.absolute()) + '/plugins/',
                '--simulate',
                '-f',
                plugin_file]
        process = subprocess.run(args,
                                 stdout=subprocess.PIPE)
        if process.returncode != 0:
            self.logger.error('------ERROR: parsing docstring for apidocs in plugin ' + plugin_file)
            self.logger.error('             server can not start until apidocs syntax is fixed')
            sys.exit(1)

    def generate_doc(self, timeout=1, maxtries=5):
        """
        this method allows to generated apidocs documentation parsing plugin files
        """
        self.logger.warning('-----------------------')
        self.logger.warning('------ Creating documentation')

        rmtree(self.apidoc_static_dir, ignore_errors=True)
        args = ['apidoc', '-c', self.apidoc_config_dir, '-i',
                str(pathlib.Path(__file__).parent.absolute()) + '/plugins/']

        for plugin in self.plugins:
            self.check_apidoc_syntax(plugin['path'])
            args.append('-f')
            args.append(plugin['path'])
        args.append('-o')
        args.append(self.apidoc_output_dir)
        process = subprocess.Popen(args,
                                   stdout=subprocess.PIPE,
                                   stderr=subprocess.STDOUT)
        counter = 0
        while process.poll() is None:
            time.sleep(timeout)
            counter = counter + 1
            if counter == maxtries:
                process.kill()
                break
        self.logger.warning(
            '------ Apidocs at http://{}:{}/apidoc/index.html'.format(self.ip, self.port))

    def start(self):
        """
        Method to start server
        """
        self.app.run(host=self.ip, port=self.port, debug=True)



from hunabku.HunabkuBase import HunabkuPluginBase, endpoint


class Hello(HunabkuPluginBase):
    def __init__(self, hunabku):
        super().__init__(hunabku)

    @endpoint('/hello', methods=['GET'])
    def hello(self):
        """
        @api {get} /hello/:id Simple Hello with ID
        @apiName Hello
        @apiGroup Hello

        @apiParam {Number} id Users unique ID.

        @apiSuccess {String} firstname Firstname of the User.
        @apiSuccess {String} lastname  Lastname of the User.
        """
        if self.valid_apikey():
            response = self.app.response_class(
                response=self.json.dumps({'hello': 'world'}),
                status=200,
                mimetype='application/json'
            )
            return response
        else:
            return self.apikey_error()



from hunabku.HunabkuBase import HunabkuPluginBase, endpoint
from flask import (
    render_template,
)


class ApiDoc(HunabkuPluginBase):
    def __init__(self, hunabku):
        super().__init__(hunabku)

    @endpoint('/apidoc', methods=['GET'])
    def index(self):
        return render_template('apidoc.html', apidoc_link='/apidoc/index.html')

    @endpoint('/apidoc/', methods=['GET'])
    def index_slash(self):
        return render_template('apidoc.html', apidoc_link='/apidoc/index.html')

    @endpoint('/apidoc/update', methods=['GET'])
    def update(self):
        """
        @api {get} /apidoc/update Update ApiDoc Documentation
        @apiName ApiDoc
        @apiGroup ApiDoc
        """
        if self.valid_apikey():
            self.hunabku.generate_doc()
            response = self.app.response_class(
                response=self.json.dumps({'status': 'apidoc updated'}),
                status=200,
                mimetype='application/json'
            )
            return response
        else:
            return self.apikey_error()



from __future__ import print_function
import sys
from pymongo import MongoClient
import pandas as pd
import numpy as np


class HunabkuLoader:
    def __init__(self, dbname='colav',
                 dburi='mongodb://localhost:27017/', dbdrop=False):
        self.dbname = dbname
        self.dbclient = MongoClient(dburi)
        self.db = self.dbclient[self.dbname]
        if dbdrop:
            self.dbclient.drop_database(dbname)

    def check_fields(self, data):
        columns = [
            "journal",
            "publisher",
            "country",
            "article_id",
            "title",
            "author",
            "doi",
            "year",
            "volume",
            "issue",
            "pages",
            "language",
            "abstract"]
        for col in columns:
            if col not in data.keys():
                print('{} not found'.format(col))
                sys.exit(1)

    def load(self, filename, dbcollection, drop=True):
        dbcollection = 'data'
        if filename is None:
            print("Error: file is not provided!", file=sys.stderr)
            sys.exit(1)

        if filename[len(filename) - 4:len(filename)].lower() == ".csv":
            data = pd.read_csv(filename)
            data["_id"] = np.arange(data.shape[0])
            print(data["_id"])
            data = data.to_dict(orient='records')
            self.check_fields(data)
            self.db[dbcollection].insert_many(data)
        elif filename[len(filename) - 5:len(filename)].lower() == ".json":
            data = pd.read_json(filename)
            data["_id"] = np.arange(data.shape[0])
            print(data["_id"])
            self.check_fields(data)
            data = data.to_dict(orient='records')
            self.db[dbcollection].insert_many(data)
        elif filename[len(filename) - 5:len(filename)].lower() == ".xlsx":
            data = pd.read_excel(filename)
            data["_id"] = np.arange(data.shape[0])
            print(data["_id"])
            self.check_fields(data)
            data = data.to_dict(orient='records')
            self.db[dbcollection].insert_many(data)
        else:
            print("file format " + filename[len(filename) - 4:len(filename)].lower() + " not supported", file=sys.stderr)
            sys.exit(1)



from hunabku.HunabkuBase import HunabkuPluginBase, endpoint
from bson import ObjectId


class MoaiGSStage(HunabkuPluginBase):
    def __init__(self, hunabku):
        super().__init__(hunabku)

    @endpoint('/moai/gs/stage/submit', methods=['GET'])
    def stage_submit(self):
        """
        @api {get} /moai/gs/stage/submit Submit Paper
        @apiName  Moai Stage
        @apiGroup Moai GSStage
        @apiDescription Allows to submit papers to the collection stage in the given databse db.

        @apiParam {String} db  Database to use in mongodb
        @apiParam {Object} data Json with paper data
        @apiParam {String} apikey  Credential for authentication


        @apiSuccess {String}  msg  GSLookUp paper inserted in stage
        @apiError (Error 401) msg  The HTTP 401 Unauthorized invalid authentication apikey for the target resource.
        """
        data = self.request.args.get('data')
        db = self.request.args.get('db')
        self.db = self.dbclient[db]
        if self.valid_apikey():
            jdata = self.json.loads(data)
            jdata["_id"] = ObjectId(jdata["_id"])
            self.db['stage'].insert(jdata, check_keys=False)
            response = self.app.response_class(
                response=self.json.dumps(
                    {'msg': 'GSLookUp Paper inserted in stage'}),
                status=200,
                mimetype='application/json'
            )
            return response
        else:
            return self.apikey_error()

    @endpoint('/moai/gs/stage/read', methods=['GET'])
    def stage_read(self):
        """
        @api {get} /moai/gs/stage/read Retrieve data from stage
        @apiName Moai Stage
        @apiGroup Moai GSStage
        @apiDescription Allow to retrieve data from stage, criteria to send if by chunks is not defined yet

        @apiParam {String} db  Database to use in mongodb
        @apiParam {String} ids  Paper ids to retrieve
        @apiParam {String} apikey  Credential for authentication


        @apiSuccess {Object}  json  all the registers from stage collection in a json dump

        @apiError (Error 401) msg  The HTTP 401 Unauthorized invalid authentication apikey for the target resource.
        """
        db = self.request.args.get('db')
        self.db = self.dbclient[db]

        if self.valid_apikey():
            cursor = self.db['stage'].find()

            data = list(cursor)
            response = self.app.response_class(
                response=self.json.dumps(data),
                status=200,
                mimetype='application/json'
            )
            return response
        else:
            return self.apikey_error()



from hunabku.HunabkuBase import HunabkuPluginBase, endpoint
from bson import ObjectId


class MoaiGSQuery(HunabkuPluginBase):
    def __init__(self, hunabku):
        super().__init__(hunabku)

    @endpoint('/moai/gs/query/cache/submit', methods=['GET'])
    def gsquery_cache_submit(self):
        """
        @api {get} /moai/gs/query/cache/submit Submit query cache
        @apiName GSQuery
        @apiGroup Moai GSQuery
        @apiDescription Allows to submit query cache to the collection cache_cites in the given database db.

        @apiParam {String} db  Database to use in mongodb
        @apiParam {Object} data Json with cite data
        @apiParam {String} apikey  Credential for authentication

        @apiError (Error 401) msg  The HTTP 401 Unauthorized invalid authentication apikey for the target resource.
        """
        data = self.request.args.get('data')
        db = self.request.args.get('db')
        self.db = self.dbclient[db]
        if self.valid_apikey():
            self.db['cache_queries'].insert(self.json.loads(data))
            response = self.app.response_class(
                response=self.json.dumps({}),
                status=200,
                mimetype='application/json'
            )
            return response
        else:
            return self.apikey_error()

    @endpoint('/moai/gs/query/cache/checkpoint', methods=['GET'])
    def gsquery_cache_read(self):
        """
        @api {get} /moai/gs/query/cache/checkpoint GSQuery cache checkpoint
        @apiName GSQuery
        @apiGroup Moai GSQuery
        @apiDescription Allow to know the cuerrent status of the collection cache_cites for the given db
                        Return the registers for cites not dowloaded yet.

        @apiParam {String} db  Database to use in mongodb
        @apiParam {String} apikey  Credential for authentication


        @apiSuccess {String[]}  Resgisters from cache with urls to download.

        @apiError (Error 401) msg  The HTTP 401 Unauthorized invalid authentication apikey for the target resource.
        """
        db = self.request.args.get('db')
        self.db = self.dbclient[db]

        if self.valid_apikey():
            cursor = self.db['cache_queries'].find(
                {'downloaded': 0, 'empty': 0})
            data = []
            for i in cursor:
                data.append(i)
            response = self.app.response_class(
                response=self.json.dumps(data),
                status=200,
                mimetype='application/json'
            )
            return response
        else:
            return self.apikey_error()

    @endpoint('/moai/gs/query/cache/update', methods=['GET'])
    def gsquery_cache_update(self):
        """
        @api {get} /moai/gs/query/cache/update Update GSQuery cache
        @apiName GSQuery
        @apiGroup Moai GSQuery
        @apiDescription Allow to updated query cache and check if it was downloaded or if it and empty page

        @apiParam {String} db  Database to use in mongodb
        @apiParam {String} _id  Cite id to update
        @apiParam {String} empty  Status, to check if the page is empty
        @apiParam {String} apikey  Credential for authentication

        @apiError (Error 401) msg  The HTTP 401 Unauthorized invalid authentication apikey for the target resource.
        """
        _id = self.request.args.get('_id')
        empty = self.request.args.get('empty')
        db = self.request.args.get('db')
        self.db = self.dbclient[db]

        if self.valid_apikey():
            # self.db['cache_queries'].update_one({"_id":ObjectId(_id)},{"$set":self.json.loads(data)})
            self.db['cache_queries'].update_one({'_id': ObjectId(_id)}, {
                "$set": {'downloaded': 1, 'empty': empty}})
            response = self.app.response_class(
                response=self.json.dumps({}),
                status=200,
                mimetype='application/json'
            )
            return response
        else:
            return self.apikey_error()



from hunabku.HunabkuBase import HunabkuPluginBase, endpoint


class MoaiGSCites(HunabkuPluginBase):
    def __init__(self, hunabku):
        super().__init__(hunabku)

    @endpoint('/moai/gs/cites/cache/checkpoint', methods=['GET'])
    def checkpoint_cites_endpoint(self):
        """
        @api {get} /moai/gs/cites/cache/checkpoint GSCites checkpoint
        @apiName GSCites
        @apiGroup Moai GSCites
        @apiDescription Allow to know the cuerrent status of the collection cache_cites for the given db
                        Return the registers for cites not dowloaded yet.

        @apiParam {String} db  Database to use in mongodb
        @apiParam {String} apikey  Credential for authentication


        @apiSuccess {String[]}  Resgisters from cache with urls to download.

        @apiError (Error 401) msg  The HTTP 401 Unauthorized invalid authentication apikey for the target resource.
        """
        db = self.request.args.get('db')
        self.db = self.dbclient[db]

        if self.valid_apikey():
            cursor = self.db['cache_cites'].find({'downloaded': 0, 'empty': 0})
            data = []
            for i in cursor:
                data.append(i)
            response = self.app.response_class(
                response=self.json.dumps(data),
                status=200,
                mimetype='application/json'
            )
            return response
        else:
            return self.apikey_error()

    @endpoint('/moai/gs/cites/cache/ids', methods=['GET'])
    def cites_cache_ids(self):  # this is the checkpoint for cache not for cites itself
        """
        @api {get} /moai/gs/cites/cache/ids Retrieve cache ids
        @apiName GSCites
        @apiGroup Moai GSCites
        @apiDescription Allow to download the register ids from collection cache_cites

        @apiParam {String} db  Database to use in mongodb
        @apiParam {String} ids  Paper ids to retrieve
        @apiParam {String} apikey  Credential for authentication


        @apiSuccess {Object}  json  all the register ids from cache_cites collection in a json dump

        @apiError (Error 401) msg  The HTTP 401 Unauthorized invalid authentication apikey for the target resource.
        """
        db = self.request.args.get('db')
        self.db = self.dbclient[db]

        if self.valid_apikey():
            cursor = self.db['cache_cites'].find({}, {'_id': 1})
            data = []
            for i in cursor:
                data.append(i)
            response = self.app.response_class(
                response=self.json.dumps(data),
                status=200,
                mimetype='application/json'
            )
            return response
        else:
            return self.apikey_error()

    @endpoint('/moai/gs/cites/cache/update', methods=['GET'])
    def cites_cache_update(self):
        """
        @api {get} /moai/gs/cites/cache/update Update GSCites cache
        @apiName GSCites
        @apiGroup Moai GSCites
        @apiDescription Allow to updated cites cache and check if it was downloaded or if it and empty page

        @apiParam {String} db  Database to use in mongodb
        @apiParam {String} _id  Cite id to update
        @apiParam {String} empty  Status, to check if the page is empty
        @apiParam {String} apikey  Credential for authentication

        @apiError (Error 401) msg  The HTTP 401 Unauthorized invalid authentication apikey for the target resource.
        """
        _id = self.request.args.get('_id')  # object id
        empty = self.request.args.get('empty')
        db = self.request.args.get('db')
        self.db = self.dbclient[db]
        if self.valid_apikey():
            self.db['cache_cites'].update_one({'_id': self.json.loads(_id)}, {
                                              "$set": {'downloaded': 1, 'empty': empty}})
            response = self.app.response_class(
                response=self.json.dumps({}),
                status=200,
                mimetype='application/json'
            )
            return response
        else:
            return self.apikey_error()

    @endpoint('/moai/gs/cites/submit', methods=['GET'])
    def stage_cites_submit(self):
        """
        @api {get} /moai/gs/cites/submit Submit Cite
        @apiName GSCites
        @apiGroup Moai GSCites
        @apiDescription Allows to submit cites to the collection stage_cites in the given database db.

        @apiParam {String} db  Database to use in mongodb
        @apiParam {Object} data Json with cite data
        @apiParam {String} apikey  Credential for authentication

        @apiError (Error 401) msg  The HTTP 401 Unauthorized invalid authentication apikey for the target resource.
        """
        data = self.request.args.get('data')
        db = self.request.args.get('db')
        self.db = self.dbclient[db]
        if self.valid_apikey():
            self.db['stage_cites'].insert(self.json.loads(data))
            response = self.app.response_class(
                response=self.json.dumps({}),
                status=200,
                mimetype='application/json'
            )
            return response
        else:
            return self.apikey_error()

    @endpoint('/moai/gs/cites/cache/submit', methods=['GET'])
    def cites_cache_submit(self):
        """
        @api {get} /moai/gs/cites/cache/submit Submit cites cache
        @apiName GSCites
        @apiGroup Moai GSCites
        @apiDescription Allows to submit cites cache to the collection cache_cites in the given database db.

        @apiParam {String} db  Database to use in mongodb
        @apiParam {Object} data Json with cite data
        @apiParam {String} apikey  Credential for authentication

        @apiError (Error 401) msg  The HTTP 401 Unauthorized invalid authentication apikey for the target resource.
        """
        data = self.request.args.get('data')
        db = self.request.args.get('db')
        self.db = self.dbclient[db]
        if self.valid_apikey():
            self.db['cache_cites'].insert(self.json.loads(data))
            response = self.app.response_class(
                response=self.json.dumps({}),
                status=200,
                mimetype='application/json'
            )
            return response
        else:
            return self.apikey_error()
